#!/usr/bin/env python

import os
import json

from collections import defaultdict
from argparse import ArgumentParser

from dtags.colors import PINK, CYAN, CLEAR
from dtags.completion import ChoicesCompleter, autocomplete
from dtags.config import load_tags
from dtags.utils import expand_path


def main():
    tag_to_paths = load_tags()
    path_to_tags = defaultdict(set)
    for tag, paths in tag_to_paths.items():
        for path in paths.keys():
            path_to_tags[path].add(tag)

    parser = ArgumentParser(
        prog="tags",
        description="dtags: display tags and tagged directories",
        usage="tags [options] [paths] [tags]",
    )
    parser.add_argument(
        "-e", "--expand",
        action="store_true",
        help="Expand the directory paths"
    )
    parser.add_argument(
        "-r", "--reverse",
        help="display tags to directories",
        action="store_true"
    )
    parser.add_argument(
        "-j", "--json",
        help="display the raw JSON",
        action="store_true"
    )
    parser.add_argument(
        "search_terms",
        type=str,
        nargs='*',
        metavar="[paths] [tags]",
        help="tag and/or directory paths to search",
    ).completer = ChoicesCompleter(tag_to_paths.keys())
    autocomplete(parser)
    parsed = parser.parse_args()

    # Filter by any given tags and paths
    # TODO optimize here if possible
    if not parsed.search_terms:
        if parsed.expand:
            filtered = {t: ps.keys() for t, ps in tag_to_paths.items()}
        else:
            filtered = {t: ps.values() for t, ps in tag_to_paths.items()}
    else:
        filtered = {}
        for term in parsed.search_terms:
            if term in tag_to_paths:
                if parsed.expand:
                    filtered[term] = tag_to_paths[term].keys()
                else:
                    filtered[term] = tag_to_paths[term].values()
            elif os.path.isdir(term):
                term = expand_path(term)
                if term in path_to_tags:
                    for tag in path_to_tags[term]:
                        if parsed.expand:
                            filtered[tag] = tag_to_paths[tag].keys()
                        else:
                            filtered[tag] = tag_to_paths[tag].values()

    if parsed.json:
        formatted = {tag: sorted(paths) for tag, paths in filtered.items()}
        print(json.dumps(formatted, sort_keys=True, indent=4))
    elif parsed.reverse:
        reverse = defaultdict(set)
        for tag, paths in filtered.items():
            for path in paths:
                reverse[path].add(tag)
        for path, tags in reverse.items():
            print("{}{}{}".format(CYAN, path, CLEAR))
            print("{}{}{}\n".format(PINK, " ".join(sorted(tags)), CLEAR))
    else:
        for tag, paths in sorted(filtered.items()):
            print("{}{}{}".format(PINK, tag, CLEAR))
            print("{}{}{}\n".format(CYAN, "\n".join(sorted(paths)), CLEAR))



import os
import json

from dtags.utils import halt, expand_path

TAGS = os.path.expanduser('~/.dtags')


def load_tags():
    """Load the tags from disk."""
    if not os.path.exists(TAGS):
        try:
            with open(TAGS, "w") as config_file:
                json.dump({}, config_file)
        except (IOError, OSError) as e:
            halt("Failed to init {}: {}".format(TAGS, e.strerror), e.errno)
    else:
        try:
            with open(TAGS, "r") as config_file:
                json_str = config_file.read().strip()
                tag_data = {} if not json_str else json.loads(json_str)
                return {
                    tag: {expand_path(path): path for path in paths}
                    for tag, paths in tag_data.items()
                }
        except ValueError as e:
            halt("Failed to load {}: {}".format(TAGS, e))
        except (IOError, OSError) as e:
            halt("Failed to load {}: {}".format(TAGS, e.strerror), e.errno)


def save_tags(tags):
    """Save the tags to disk.

    :param tags: tags to save
    """
    try:
        with open(TAGS, "w") as config_file:
            json.dump(
                {tag: sorted(paths.values()) for tag, paths in tags.items()},
                config_file,
                indent=4,
                sort_keys=True
            )
    except IOError as e:
        halt("Failed to save {}: {}".format(TAGS, e.strerror))
